import { Component, OnInit, ViewChild } from '@angular/core';
import { IgxDataChartComponent, IgxStackedAreaSeriesComponent, IgxStackedFragmentSeriesComponent, IgxStackedLineSeriesComponent } from "igniteui-angular-charts";
import { COLUMNS, HORIZON, SALES } from './data';

@Component({
  selector: 'app-chart-page',
  templateUrl: './chart-page.component.html',
  styleUrls: ['./chart-page.component.scss']
})
export class ChartPageComponent implements OnInit {

  @ViewChild("prodOutChart", { static: false}) public prodOutChart : IgxDataChartComponent;
  @ViewChild("procOutChart", { static: false}) public procOutChart : IgxDataChartComponent;

  activeTab:number = 0;
  columns:any[] = COLUMNS;
  compareTimePeriods:string[] = HORIZON;
  designs:string[] = [];
  height:number = window.innerHeight;
  outData1:any[] = [];
  outData2:any[] = [];
  parameterData:any[] = SALES;
  processes:string[] = [];
  scenario1:string = 'PLAN_2021_51';
  scenario2:string = 'BETA_2021_51_CS';

  constructor() { }

  // ngOnInit(): void {
    // let previousItem: any = null, index: number = (this.data1.length - 1) / 2; //Calculates the middle of the category x-axis plot area
    // for(let i=0; i<this.data1.length; i++){

    //   let item1 = this.data1[1], item2 = this.data1[2],
    //     value1 = 0, value2 = 0,
    //     label = this.seriesTitles[i];

    //   switch (label) {
    //     case "Classic AT-AT": {
    //       value1 = item1.StarWarsATAT;
    //       value2 = item2.StarWarsATAT;
    //       break;
    //     }
    //     case "First Order AT-AT": {
    //       value1 = item1.StarWarsOrderAT;
    //       value2 = item2.StarWarsOrderAT;
    //       break;
    //     }
    //     case "Avengers Tower": {
    //       value1 = item1.MarvelTower;
    //       value2 = item2.MarvelTower;
    //       break;
    //     }
    //     case "Stark's Mansion": {
    //       value1 = item1.MarvelMansion;
    //       value2 = item2.MarvelMansion;
    //       break;
    //     }
    //   }

    //   let calloutValue = (value1 + value2) / 2; //The midpoint between the two points. Needed because index is fractional. Not needed if it's not fractional
    //   calloutValue = previousItem ? calloutValue + previousItem.Value : (calloutValue / 2) - 1;

    //   previousItem = { Index: index, Value: calloutValue, Label: this.seriesTitles[i] };
    //   this.calloutData.push(previousItem);
    // }
  // }

  ngOnInit(): void {
    this.parameterData.forEach((d) => { //build designs/processes arrays
      if (this.designs.indexOf(d.Design) < 0 && d.Design !== undefined) { this.designs.push(d.Design); }
      if (this.processes.indexOf(d.Process) < 0 && d.Process !== undefined) { this.processes.push(d.Process); }
    });
  }

  itemSelected (e:any) {
    this.activeTab = e;
    this.populateChart();
  }

  populateChart() {
    if (this.activeTab === 1) {
      this.buildProdOutData();
      this.prodOutChart.series.clear();
      this.resetOutChartData(this.prodOutChart, this.designs);
    } else if (this.activeTab === 2) {
      this.buildProcOutData();
      this.procOutChart.series.clear();
      this.resetOutChartData(this.procOutChart, this.processes);
    }
  }

  resetOutChartData(chart:IgxDataChartComponent, filter:string[]) {
    let areaSeries: IgxStackedAreaSeriesComponent = chart.actualSeries[0] as IgxStackedAreaSeriesComponent,
        lineSeries: IgxStackedLineSeriesComponent = chart.actualSeries[1] as IgxStackedLineSeriesComponent;

    //These two lines will clear the existing fragments
    areaSeries.series.clear();
    lineSeries.series.clear();

    filter.forEach((f) => {
      let areaFrag: IgxStackedFragmentSeriesComponent = new IgxStackedFragmentSeriesComponent(),
        lineFrag: IgxStackedFragmentSeriesComponent = new IgxStackedFragmentSeriesComponent();

      areaFrag.valueMemberPath = f;
      areaFrag.title = f;
      areaSeries.series.add(areaFrag);
      lineFrag.valueMemberPath = f;
      lineFrag.title = f;
      lineFrag.thickness = 3;
      lineSeries.series.add(lineFrag);
    });

    areaSeries.renderSeries(false);
    lineSeries.renderSeries(false);
  }

  buildProdOutData() {
    let scen1Data: any[] = this.parameterData.filter((d) => d.Scenario === this.scenario1 && d.Design !== undefined),
        scen2Data: any[] = this.parameterData.filter((d) => d.Scenario === this.scenario2 && d.Design !== undefined);
    this.outData1 = [], this.outData2 = [];

    scen1Data.forEach((d, j) => {
      let design: string = d.Design;
      this.compareTimePeriods.forEach((p, i) => {
        let out1:number = scen2Data[j][p] > 0 ? scen2Data[j][p] : 0, out2:number = d[p] > 0 ? d[p] : 0;

        if (this.outData1[i] === undefined) { this.outData1[i] = { Date: p }; }
        if (this.outData2[i] === undefined) { this.outData2[i] = { Date: p }; }

        if (design in this.outData1[i]) { this.outData1[i][design] += out1; }
        else { this.outData1[i][design] = out1; }

        if (design in this.outData2[i]) { this.outData2[i][design] += out2; }
        else { this.outData2[i][design] = out2; }
      });
    });
  }

  buildProcOutData() {
    let scen1Data: any[] = this.parameterData.filter((d) => d.Scenario === this.scenario1 && d.Design !== undefined),
        scen2Data: any[] = this.parameterData.filter((d) => d.Scenario === this.scenario2 && d.Design !== undefined);
    this.outData1 = [], this.outData2 = [];

    scen1Data.forEach((d, j) => {
      let process: string = d.Process;
      this.compareTimePeriods.forEach((p, i) => {
        let out1:number = scen2Data[j][p] > 0 ? scen2Data[j][p] : 0, out2:number = d[p] > 0 ? d[p] : 0;

        if (this.outData1[i] === undefined) { this.outData1[i] = { Date: p }; }
        if (this.outData2[i] === undefined) { this.outData2[i] = { Date: p }; }

        if (process in this.outData2[i]) { this.outData2[i][process] += out2; }
        else { this.outData2[i][process] = out2; }

        if (process in this.outData1[i]) { this.outData1[i][process] += out1; }
        else { this.outData1[i][process] = out1; }
      });
    });
  }
}
